// -*- C++ -*-
/*!
 * @file  ControlRisk.cpp * @brief ControlRisk * $Date$ 
 *
 * $Id$ 
 */
#include "ControlRisk.h"
#include "ControlRiskDialog.h"

// for XML
#include <iostream>
using namespace std;

#include "SerDesIF.h"
#include "GridMap.h"


const char ControlRisk::Category[] = "navigation";
const char ControlRisk::Name[]     = "control_risk";

// Module specification
// <rtc-template block="module_spec">
static const char* envrecognition_spec[] =
  {
    "implementation_id", "ControlRisk",
    "type_name",         "ControlRisk",
    "description",       "ControlRisk",
    "version",           "1.0.0",
    "vendor",            "Chiba Institute of Technology",
    "category",          "Sensor",
    "activity_type",     "SPORADIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.filename", "./ControlRisk.xml",
    ""
  };
// </rtc-template>

ControlRisk::
ControlRisk(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    //m_inIn("in", m_in),
    m_outOut("out", m_out)

    // </rtc-template>
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  //registerInPort("in", m_inIn);

  // Set OutPort buffer
  registerOutPort("out", m_outOut);

  // Set service provider to Ports

  // Set service consumers to Ports

  // Set CORBA Service Ports

  // </rtc-template>
	
	Config     = NULL;
	Result     = NULL;
	EnvData    = NULL;
	PosData    = NULL;
	ObjectData = NULL;
	RobotData  = NULL;
}

ControlRisk::
~ControlRisk()
{
	vector<RTC::TimedString *>::iterator it_m_in;
	vector<RTC::InPort<RTC::TimedString> *>::iterator it_m_inIn;

	if(RobotData != NULL){
		delete RobotData;
	}

	if(ObjectData != NULL){
		delete ObjectData;
	}

	if(PosData != NULL){
		delete PosData;
	}

	if(EnvData != NULL){
		delete EnvData;
	}

	if(Result != NULL){
		delete Result;
	}

	if(Config != NULL){
		delete Config;
	}

	it_m_in = m_in.begin();
	while(it_m_in != m_in.end()){
		delete *it_m_in;
		it_m_in++;
	}

	it_m_inIn = m_inIn.begin();
	while(it_m_inIn != m_inIn.end()){
		delete *it_m_inIn;
		it_m_inIn++;
	}
}


RTC::ReturnCode_t ControlRisk::
onInitialize()
{
	int i;
	int ret;

	bindParameter( "filename", filename, "./ControlRisk.xml");

	try{
		Config = new ControlRiskConfig();
	}catch(...){
		cerr << "can't allocate memory for config." << endl;
		return RTC::RTC_ERROR;
	}
	ret = Config->LoadFromFile(filename);
	if(ret != 0){
		cerr << "can't load config file \"" << filename << "\"" << endl;
		delete Config;
		Config = NULL;
		return RTC::RTC_ERROR;
	}

	for(i = 0; i < Config->input_number; i++){
		char buf[10];
		m_in.push_back(new RTC::TimedString);
		sprintf(buf, "in%d", i);
		m_inIn.push_back(new RTC::InPort<RTC::TimedString>(buf, *m_in[i]));
		registerInPort(buf, *m_inIn[i]);
	}
	
	return RTC::RTC_OK;
}

pthread_t tid;

RTC::ReturnCode_t ControlRisk::
onActivated(RTC::UniqueId ec_id)
{
	int ret;

	Config = new ControlRiskConfig;

	ret = Config->LoadFromFile(filename);
	if(ret != 0){
		cerr << "can't load config file \"" << filename << "\"" << endl;
		delete Config;
		Config = NULL;
		return RTC::RTC_ERROR;
	}

	// 設定ファイル内のカテゴリ名とコンポーネント名をチェック
	if(Config->category != ControlRisk::Category){
		cerr << "Category NOT match (" << Config->category << " != " << ControlRisk::Category << ")." << endl;
		delete Config;
		Config = NULL;
		return RTC::RTC_ERROR;
	}
	if(Config->name != ControlRisk::Name){
		cerr << "Name NOT match (" << Config->name << " != " << ControlRisk::Name << ")." << endl;
		delete Config;
		Config = NULL;
		return RTC::RTC_ERROR;
	}

	// リスクコントロール初期化
	estimateRisk.init();
//	estimateRisk.setMemoryPara(memory_period, memory_decrease);

	// GUI 初期化
	if(Config->gui){
		pthread_create(&tid,NULL,windowThread,NULL);
	}

	// データ送信用インタフェイスクラス初期化
	if(Result != NULL){
		delete Result;
	}

	Result = new SerDesIF<GridMap>(ControlRisk::Category, ControlRisk::Name, Config->id);


	Result->Data = Config->grid_map;

	Result->Data.map.clear();
	Result->Data.resize();
	//memset(&(Result->grid_map.map[0]), 0, x * y * sizeof(float));

	// データ受信用インタフェイスクラス初期化
	EnvData    = new SerDesIF<GridMap>();
	PosData    = new SerDesIF<Localization>();
	ObjectData = new SerDesIF<DetectObject>();
	RobotData  = new SerDesIF<RobotParam>();

	return RTC::RTC_OK;
}

RTC::ReturnCode_t ControlRisk::
onDeactivated(RTC::UniqueId ec_id)
{
	// GUI終了処理
	if(Config->gui){
		close_window();
		pthread_join(tid, NULL);
	}

	// リスクコントロール初期化
	estimateRisk.close();

	// データ受信用インタフェイスクラスの終了処理
	if(RobotData != NULL){
		delete RobotData;
		RobotData = NULL;
	}

	if(ObjectData != NULL){
		delete ObjectData;
		ObjectData = NULL;
	}

	if(PosData != NULL){
		delete PosData;
		PosData = NULL;
	}

	if(EnvData != NULL){
		delete EnvData;
		EnvData = NULL;
	}

	// データ送信用インタフェイスクラスの終了処理
	if(Result != NULL){
		delete Result;
		Result = NULL;
	}

	if(Config != NULL){
		delete Config;
		Config = NULL;
	}

	return RTC::RTC_OK;
}

RTC::ReturnCode_t ControlRisk::onExecute(RTC::UniqueId ec_id)
{
	int i;
	string recv;
	baseRTC::Header header;
	
	for(i=0; i<Config->input_number; i++){
		if (m_inIn[i]->isNew()){
			m_inIn[i]->read();
			recv = m_in[i]->data;

			cerr << recv << endl; // 受信XML表示

			header.readFromXml(recv); // 受信XMLから, ヘッダを取り出す

			// 受信したデータのIDが, 設定ファイルのreceive 項目にあれば処理
			if(find(Config->receive.begin(), Config->receive.end(), header.id) != Config->receive.end()){
				// ヘッダ内のデータ情報から受信データの型を取得
				if((header.sender_name == "env_recognition") && (header.type == GridMap::TYPE_ID)){
					ProcessEnvRecognitionData(recv);
				}else if((header.sender_name == "env_recognition") && (header.type == DetectObject::TYPE_ID)){
					ProcessDetectObjectData(recv);
				}else if(header.sender_name == "position_estimate"){
					ProcessSelfLocationData(recv);
				}else if(header.sender_name == "control_motor"){
					ProcessRobotParamData(recv);
				}
			}else{
				cerr << "Received data is NOT exist in receive list (" << header.id << ")." << endl;
			}

		}
	}

	return RTC::RTC_OK;
}

RTC::ReturnCode_t ControlRisk::
ProcessEnvRecognitionData(string &recv)
{
	int ret;

	ret = EnvData->DeserializeFromXML(recv); // 受信XMLからデータ復元
	if(ret != 0){ // 復元失敗
		cerr << "Data deserialize failed (EnvRegcognitionData)[" << ret << "]." << endl;
		return RTC::RTC_ERROR;
	}

	estimateRisk.setGridMap(&EnvData->Data);
	estimateRisk.getRiskGridMap(&(Result->Data));
	Result->Data.time = EnvData->Data.time;

	SendResult();

	if(Config->gui){
		set_value(&Result->Data);
	}

	return RTC::RTC_OK;
}

RTC::ReturnCode_t ControlRisk::
SendResult(void)
{
	struct timeval tv;
	string xml;

	gettimeofday(&tv,NULL);
	m_out.tm.sec = tv.tv_sec;
	m_out.tm.nsec = tv.tv_usec*1000;

	Result->SerializeToXML(&xml);

	cout << "===== XML to send. =====" << endl;
	cout << xml;
	cout << "========================" << endl;

	m_out.data = xml.c_str();
	m_outOut.write();

	return RTC::RTC_OK;
}

RTC::ReturnCode_t ControlRisk::
ProcessSelfLocationData(string &recv)
{
	int ret;

	ret = PosData->DeserializeFromXML(recv); // 受信XMLからデータ復元
	if(ret != 0){ // 復元失敗
		cerr << "Data deserialize failed (PositionEstimateData)[" << ret << "]." << endl;
		return RTC::RTC_ERROR;
	}

	estimateRisk.setSelfLocalization(&PosData->Data);

	return RTC::RTC_OK;
}

RTC::ReturnCode_t ControlRisk::
ProcessDetectObjectData(string &recv)
{
	int ret;

	ret = ObjectData->DeserializeFromXML(recv); // 受信XMLからデータ復元
	if(ret != 0){ // 復元失敗
		cerr << "Data deserialize failed (ObjectData)[" << ret << "]." << endl;
		return RTC::RTC_ERROR;
	}

	// 障害物登録
	estimateRisk.setObject(&ObjectData->Data);

	return RTC::RTC_OK;
}

RTC::ReturnCode_t ControlRisk::
ProcessRobotParamData(string &recv)
{
	int ret;

	ret = RobotData->DeserializeFromXML(recv); // 受信XMLからデータ復元
	if(ret != 0){ // 復元失敗
		cerr << "Data deserialize failed (RobotPram)[" << ret << "]." << endl;
		return RTC::RTC_ERROR;
	}

	// ロボット形状登録
	estimateRisk.setRobotPara(&RobotData->Data);

	return RTC::RTC_OK;
}

extern "C"
{
 
  void ControlRiskInit(RTC::Manager* manager)
  {
    RTC::Properties profile(envrecognition_spec);
    manager->registerFactory(profile,
                             RTC::Create<ControlRisk>,
                             RTC::Delete<ControlRisk>);
  }
  
};
