// -*- C++ -*-
/*!
 * @file  ControlRisk.h * @brief ControlRisk * @date  $Date$ 
 *
 * $Id$ 
 */
#ifndef CONTROL_RISK_H
#define CONTROL_RISK_H

#include <vector>

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>

#include "baseRTC.h"
#include "ControlRiskConfig.h"

#include "SerDesIF.h"
#include "GridMap.h"
#include "Localization.h"
#include "Object.h"

#include "EstimateRisk.h"

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>


#define NAME_LENGTH 128

class ControlRisk  : public RTC::DataFlowComponentBase
{
public:
	ControlRisk(RTC::Manager* manager);
	~ControlRisk();

	// The initialize action (on CREATED->ALIVE transition)
	// formaer rtc_init_entry() 
	virtual RTC::ReturnCode_t onInitialize();

	// The activated action (Active state entry action)
	// former rtc_active_entry()
	virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

	// The deactivated action (Active state exit action)
	// former rtc_active_exit()
	virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

	// The execution action that is invoked periodically
	// former rtc_active_do()
	virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

	static const char Category[];
	static const char Name[];

 protected:
	// Configuration variable declaration
	// <rtc-template block="config_declare">
  
	// </rtc-template>

	// DataInPort declaration
	// <rtc-template block="inport_declare">
	vector<RTC::TimedString *> m_in;
	vector<RTC::InPort<RTC::TimedString> *> m_inIn;

	// </rtc-template>

	// DataOutPort declaration
	// <rtc-template block="outport_declare">
	RTC::TimedString m_out;
	RTC::OutPort<RTC::TimedString, RTC::NullBuffer> m_outOut;

	// </rtc-template>

	// CORBA Port declaration
	// <rtc-template block="corbaport_declare">

	// </rtc-template>

	// Service declaration
	// <rtc-template block="service_declare">

	// </rtc-template>

	// Consumer declaration
	// <rtc-template block="consumer_declare">

	// </rtc-template>

private:
	/** 外界認識からの受信データ (XML) を処理する
	 *
	 * 外界認識から受信した環境データを処理する
	 *
	 * @param[in] recv 受信したXML文字列 (コピーコスト削減のため参照渡し)
	 * @return 成功時は, RTC::RTC_OKを返す. それ以外の場合は, エラーコードを返す.
	 */
	RTC::ReturnCode_t ProcessEnvRecognitionData(std::string &recv);

	/** リスクコントロールの結果を送信する
	 *
	 * 計算結果をXMLに変換して, 次のコンポーネントへ送信する
	 *
	 * @return 成功時は, RTC::RTC_OKを返す. それ以外の場合は, エラーコードを返す.
	 */
	RTC::ReturnCode_t SendResult(void);

	/** 自己位置推定からの受信データ (XML) を処理する
	 *
	 * 自己位置推定から受信した環境データを処理する
	 *
	 * @param[in] recv 受信したXML文字列 (コピーコスト削減のため参照渡し)
	 * @return 成功時は, RTC::RTC_OKを返す. それ以外の場合は, エラーコードを返す.
	 */
	RTC::ReturnCode_t ProcessSelfLocationData(std::string &recv);

	/** 外界認識からの障害物データ (XML) を処理する
	 *
	 * 外界認識から受信した障害物データを処理する
	 *
	 * @param[in] recv 受信したXML文字列 (コピーコスト削減のため参照渡し)
	 * @return 成功時は, RTC::RTC_OKを返す. それ以外の場合は, エラーコードを返す.
	 */
	RTC::ReturnCode_t ProcessDetectObjectData(std::string &recv);

	/** モータ制御からの障害物データ (XML) を処理する
	 *
	 * モータ制御から受信したロボット形状データを処理する
	 *
	 * @param[in] recv 受信したXML文字列 (コピーコスト削減のため参照渡し)
	 * @return 成功時は, RTC::RTC_OKを返す. それ以外の場合は, エラーコードを返す.
	 */
	RTC::ReturnCode_t ProcessRobotParamData(std::string &recv);


	ControlRiskConfig    *Config;  ///< 設定ファイルのデータクラス

	SerDesIF<GridMap>      *Result;     ///< 送信する結果用クラス
	SerDesIF<GridMap>      *EnvData;    ///< 受信した環境情報
	SerDesIF<Localization> *PosData;    ///< オドメトリ受信用クラス
	SerDesIF<DetectObject> *ObjectData; ///< 障害物データ受信クラス
	SerDesIF<RobotParam>   *RobotData;  ///< ロボットパラメータデータ受信クラス

	std::string filename;

	CEstimateRisk estimateRisk;
};


extern "C"
{
  void ControlRiskInit(RTC::Manager* manager);
};

#endif // CONTROL_RISK_H
