// -*- C++ -*-
/*!
 * @file EnvRecognitionComp.cpp
 * @brief Standalone component
 * @date $Date$
 *
 * $Id$
 */

/*!
	@mainpage 環境認識コンポーネント機能仕様

<h2>概要</h2>

複数個のURGからのスキャンデータを受信し, それらのデータを統合して一つの存在確率マップを作成する.
時間経過を考慮のいれ, 古いデータの信用度は低いものとして扱い, 存在確率を減らして統合する.

<h2>入出力</h2>

<h3>ポート構成</h3>

本コンポーネントのポート構成は以下の通り

<table align="center" cellpadding="0" border=1>
	<tr>
		<th bgcolor="#aaaaff" colspan=4><div align="center">ポートの構成</div></th>
	</tr>
	<tr>
		<th> 名称</th>
		<th> ポートの種類</th>
		<th> データの型</th>
		<th> 説明</th>
	</tr>
	<tr>
		<td> in<em>n</em> (nは数字) </td>
		<td> 入力ポート </td>
		<td> TimedString</td>
		<td>
			XMLにシリアライズされたオブジェクトを受信するポート.
			設定ファイルにて個数を変更可能（標準は３ポート）.
			接続位置は任意に選択可．
		</td>
	</tr>
	<tr>
		<td> out </td>
		<td> 出力ポート </td>
		<td> TimedString</td>
		<td>
			このコンポーネントの処理結果を出力するポート.
			出力データはXMLにシリアライズされて送信されるため, テキスト型である.
		</td>
	</tr>
</table>

<h3>入出力データ仕様</h3>

入力ポート(@a in) および出力ポート(@a out)では, データはXMLにシリアライ
ズされて送信される. シリアライズ／デシリアライズには<a
href="http://www.boost.org/">Boostライブラリ</a>1.34.1を使用している.
送受信のデータフォーマットについては以下のファイル内のデータ定義を参照
のこと.

<table>
	<tr>
		<th bgcolor="aaaaff" colspan=4><div align="center">入力</div></th>
	</tr>
	<tr>
		<th>入力元</th>
		<th>データの型</th>
		<th>XML</th>
		<th>説明</th>
	</tr>

	<tr>
		<td>URGコンポーネント</td>
		<td>GridMap.h</td>
		<td><a href="inputdata.html#GridMap0">GridMap</a></td>
		<td>URGでスキャンした, 周囲の障害物存在確率マップ.</td>
	</tr>
	<tr>
		<td>自己位置推定コンポーネント</td>
		<td>Localization.h</td>
		<td><a href="inputdata.html#Localization0">Localization</a></td>
		<td>GPSによる位置データとオドメトリから推定された, 現在位置のデータ.</td>
	</tr>

	<tr>
		<th bgcolor="aaaaff" colspan=4><div align="center">出力</div></th>
	</tr>
	<tr>
		<th>出力先</th>
		<th>データの型</th>
		<th>XML</th>
		<th>説明</th>
	</tr>

	<tr>
		<td>リスク管理コンポーネント</td>
		<td>GridMap.h</td>
		<td><a href="outputdata.html">GridMap</a></td>
		<td>複数のURGから受信した存在確率マップを統合した, 一つの存在確率マップ.</td>
	</tr>

</table>

本コンポーネントが入出力するデータ形式は以下の通りである. ただし, 構造
体名の部分はそれぞれXMLにシリアライズされた文字列とする.

<pre>

data = header, object-data;

header = @a baseRTC::Header;

object-data = @a GridMap

            | @a Localization

            ;
</pre>

XMLにシリアライズした入出力データに関しては以下を参照する．
<ul>
	<li><a href="inputdata.html">入力データ</a>
	<li><a href="outputdata.html">出力データ</a>
</ul>

<h2>設定</h2>
コンポーネントは，XML形式の<a href="configrationfile.html">設定ファイル</a>を参照する．

<h2>使用方法 </h2>

<h3> 開発／動作環境 </h3>

開発／動作チェックは<a href="http://www.ubuntu.com/">Ubuntu</a> 8.04.3 にて行った.

<h3> 起動／操作 </h3>

起動／操作順序は以下の通り. 以下の方法はEclipse上のRTCLinkで行うことを想定している.
<ol>
	<li>事前準備
		<ul>
			<li>OmniORBネームサービスの設定が適切に行われていること</li>
		</ul>
	</li>
	<li><a href="configrationfile.html">設定ファイル</a>を必要に応じて変更する
		<ul>
			<li>接続するコンポーネントがrecieveで受け入れ可能になっているか確認する</li>
		</ul>
	</li>
	<li>コンポーネントのバイナリを実行する
		<ul>
			<li>バイナリの置き場所に移動</li>
			<li>$ ./EnvRecognitionComp </li>
		</ul>
	</li>
	<li>ポートを接続する.
		<ul>
			<li>以下は接続例
		</ul>
		@image html  connection.png "ポート接続の例"
		<br>
		<ul>
			<li><a href="inputdata.html#GridMap0">URGコンポーネント2つからの出力</a>を, 環境認識コンポーネントの入力へ</li>
			<li><a href="inputdata.html#Localization0">自己位置推定コンポーネントからの出力</a>を, 環境認識コンポーネントの入力へ<br></li>
			    (自己位置認識コンポーネントを使用しない場合は, モータ制御コンポーネントからの出力を, 環境認識コンポーネントの入力へ)</li>
			<li><a href="outputdata.html">環境認識コンポーネントからの出力</a>を, リスク管理コンポーネントへ</li>
		</ul>
	</li>
	<li>コンポーネントをアクティベートする.</li>
</ol>

<h3>終了</h3>

終了順序は以下の通り
<ol>
	<li>本コンポーネントをディアクティベートする</li>
	<li>コンポーネントを終了する</li>
</ol>


<h2>ビルド方法 </h2>

<h3> 要求するライブラリ</h3>

ビルドに必要となるライブラリは以下の通り. Boost, Xerces-C++およびQtは
ubuntu8.04から<tt>apt-get</tt>でインストールしたものである.
<ul>
	<li><a href="http://www.is.aist.go.jp/rt/OpenRTM-aist/html/">OpenRTM-aist-0.4.2</a>及び関連パッケージ一式</li>
	<li><a href="http://www.boost.org/">Boost</a> 1.34.1</li>
	<li><a href="http://xerces.apache.org/xerces-c/">Xerces-C++ XML Parser</a> 2.8.0</li>
	<li><a href="http://qt.nokia.com/">Qt 4</a> 4.3.4</li>
</ul>

<h3> 構築方法</h3>

<ol>
	<li>
		共通ライブラリの構築を行う
		<ol>
			<li>ライブラリのあるディレクトリに移動: <tt>$ cd common/src</tt></li>
			<li>ライブラリの構築: <tt>$ make && make install </tt></li>
		</ol>
	</li>
	<li>コンポーネントの構築を行う
		<ol>
			<li>
				コンポーネントのソースファイルがあるディレクトリに移動:
				<tt>$ cd src</tt>
			</li>
			<li>
				コンポーネントの構築:
				<tt>$ make </tt>
			</li>
		</ol>
	</li>
</ol>

\page configrationfile 設定ファイル

<h1>環境認識コンポーネント設定ファイル</h1>

<h2>概要</h2>

環境認識コンポーネントの挙動を変更する設定ファイルの仕様を示す．<br>
入力ポート数，入力ポートに接続可能なコンポーネント名，存在確率を計算する領域など指定する．<br>

<h2>ファイル名</h2>

実行ファイル EnvRecognitionComp と同じディレクトリにある EnvRecognition.xml がデフォルトである．<br>
Configration の @a EnvRecognition::filename で指定しており，RT System Editor などで変更可能である．<br>

<h2>ファイルの形式</h2>

ファイルはXML形式であり，テキストエディタなどで変更する．<br>

<h2>読み込みのタイミング</h2>

設定ファイルは，コンポーネントの Activate 毎に読み込まれる．<br>
ただし， EnvRecognition::filename からのファイル名の読み込みと，入力ポート数（可変）の変更は， Initialize 時に行う．

<h2>設定ファイルの例</h2>

<pre>
<?xml version="1.0"?>
<rtc_config category="navigation" name="env_recognition">
  <id>ENV_RECOGNITION0</id>
  <receive name="TOP_URG" id="TOP_URG0"></receive>
  <receive name="TOP_URG" id="TOP_URG1"></receive>
  <receive name="PositionEstimate" id="POSITION_ESTIMATE0"></receive>
  <receive name="control_motor" id="ROBOT0"></receive>
  <input_number>3</input_number>
  <timeout>1.0</timeout>
  <work_area>-5.0,-5.0,5.0,5.0</work_area>
  <work_grid_size>0.05,0.05</work_grid_size>
  <area>-2.0,-2.0,2.0,2.0</area>
  <grid_size>0.05,0.05</grid_size>
  <range>0.0,1.0,0.1</range>
  <memory_period>2.0</memory_period>
  <memory_decrease>0.5</memory_decrease>
  <gui>ON</gui>
</rtc_config>
</pre>

<h2>パラメータの解説</h2>

<h3>rtc_config category, name</h3>
<ul>
<li>コンポーネントのカテゴリ(navigation)と名前(env_recognition)
<li>コンポーネント固有の値であるため変更しない．
</ul>

<h3>id</h3>
<ul>
<li>コンポーネントのID
<li>出力先のコンポーネントで入力元のコンポーネントを特定するために使用される．
</ul>

<h3>recieve name, id</h3>
<ul>
<li>入力ポートに接続可能なコンポーネントの名前とID
<li>リストにないコンポーネントを接続してもエラーメッセージを出力して，無視する．
</ul>

<h3>input number</h3>
<ul>
<li>入力ポート数
<li>この数に従い，Initialize 時に入力ポートの数を設定する．
<li>標準的には３ポートで，以下のコンポーネントからデータを入力している．
<li>urg x 2 URGコンポーネント
<li>position_estimate 自己位置推定コンポーネント
</ul>

<h3>work_area</h3>
<ul>
<li>存在確率を計算するグリッドマップの領域 x0, y0, x1, y1 [m]
<li>領域を広げると計算に時間がかかるため注意する．
</ul>

<h3>work_grid_size</h3>
<ul>
<li>存在確率を計算するグリッドマップのグリッドサイズ x, y [m] 
<li>グリッドのサイズを小さくすると計算に時間がかかるため注意する． 
</ul>

<h3>area</h3>
<ul>
<li>出力先のコンポーネントに送信するグリッドマップの領域 x0, y0, x1, y1 [m]
<li>領域を広げると通信に時間がかかるため注意する．
</ul>

<h3>grid_size</h3>
<ul>
<li>出力先のコンポーネントに送信するグリッドマップのグリッドサイズ x, y [m] 
<li>グリッドのサイズを小さくすると通信に時間がかかるため注意する． 
</ul>

<h3>range</h3>
<ul>
<li>出力先のコンポーネントに送信するグリッドマップの計算の最大値，最小値，ステップ 
<li>グリッドマップのデータを圧縮するために，最大値，最小値，ステップを指定する．
<li>0.0,1.0,0.1の場合は11段階のため，4ビットのデータとして扱われる．
</ul>

<h3>memory_period</h3>
<ul>
<li>存在確率を保存する時間 [sec]
<li>センサで検出した存在確率は，memory_period[s]だけ保存され参照される．
</ul>

<h3>memory_decrease</h3>
<ul>
<li>存在確率が減少する比率 [s<sup>-1</sup>]
<li>センサで検出した存在確率は，時間の経過と共に未検出(0.5)に近づくが，その減少する比率を指定する．
<li>0.5[s<sup>-1</sup>]の場合，2.0[s]で未検出(0.5)となる．
</ul>

<h3>gui</h3>
<ul>
<li>GUIを表示/非表示のフラグ ON/OFF
<li>GUIを表示すると，処理の周期が低下することがあるため注意する．
</ul>

\page inputdata 入力データ

<h1>環境認識コンポーネント入力データ</h1>

<h2>概要</h2>
環境認識コンポーネントには，以下の３つのデータを入力することが必要である．

<ol>
<li><a href="#GridMap0">存在確率を示すグリッドマップ</a>
<li><a href="#Localization0">自己位置データ</a>
</ol>

<hr>

<h2 id="gridmap0">存在確率を示すグリッドマップ</h2>
GridMap.h の構造体 GridMap をXMLにシリアライズしたデータを入力する．

<h3>入力データGridMapの例</h3>

<pre>
<?xml version="1.0" encoding="UTF-8" standalone="yes" ?>
<!DOCTYPE boost_serialization>
<boost_serialization signature="serialization::archive" version="5">
<XmlSerDesHeader class_id="0" tracking_level="0" version="0">
	<category>sensor</category>
	<type>GridMap</type>
	<sender_name>TOP_URG</sender_name>
	<id>TOP_URG1</id>
	<tag></tag>
	<time>1293871872.950712</time>
	<hdr_version>0</hdr_version>
</XmlSerDesHeader>
</boost_serialization>

<?xml version="1.0" encoding="UTF-8" standalone="yes" ?>
<!DOCTYPE boost_serialization>
<boost_serialization signature="serialization::archive" version="5">
<GridMap class_id="0" tracking_level="0" version="0">
	<time>1293871872.950712</time>
	<id>TOP_URG1</id>
	<coordinate>self</coordinate>
	<area_x0>-1</area_x0>
	<area_y0>-2</area_y0>
	<area_x1>3</area_x1>
	<area_y1>2</area_y1>
	<size_x>0.050000001</size_x>
	<size_y>0.050000001</size_y>
	<range_min>0</range_min>
	<range_max>1</range_max>
	<range_step>0.1</range_step>
	<map class_id="1" tracking_level="0" version="0">
		<range_min>0</range_min>
		<range_max>1</range_max>
		<n_values_>10</n_values_>
		<n_elems>6400</n_elems>
		<n_bits>4</n_bits>
		<packed_bytes class_id="2" tracking_level="0" version="0">
			<n_bytes>3200</n_bytes>
			<byte_stream>
ERERERERERERERERERERERERERERERERERERERERERERERERERERERERERERERERERERERER

　　　　　　　　　　　　　　＜途中省略＞

ERERERERERERERERERERERERERERERERERERERERERERERERERERERERERERERERERERERER
ERERERFERERERERERERERERERERERERERERERBERERERERERERERERERERERRERERERERERE
REREREREREREREREREQ=
			</byte_stream>
		</packed_bytes>
	</map>
</GridMap>
</boost_serialization>
</pre>

<h3>主なパラメータの解説</h3>

<h4>area_x0, area_y0, area_x1, area_y1</h4>
<ul>
<li>存在確率を表すグリッドマップの領域 x0, y0, x1, y1 [m]
</ul>

<h4>size_x, size_y</h4>
<ul>
<li>存在確率を表すグリッドマップのグリッドのサイズ x, y [m] 
</ul>

<h4>range_min, range_max, range_step</h4>
<ul>
<li>存在確率を表すグリッドマップの最大値，最小値，ステップ 
<li>グリッドマップのデータを圧縮するため．
<li>0.0,1.0,0.1の場合は11段階のため，4ビットのデータとして扱われる．
</ul>

<h4>byte_stream</h4>
<ul>
<li>存在確率を表すグリッドマップ
<li>グリッドマップのデータを上記のように圧縮した後，Base64を用いてテキストに変換した値
</ul>

その他のパラメータに関しては，実装中及び将来の拡張性のために用意している．

<hr>

<h2 id="Localization0">自己位置データ</h2>
Localization.h のクラス Localization をXMLにシリアライズしたデータを入力する．

<h3>入力データ Localization の例</h3>

<pre>
<?xml version="1.0" encoding="UTF-8" standalone="yes" ?>
<!DOCTYPE boost_serialization>
<boost_serialization signature="serialization::archive" version="5">
<XmlSerDesHeader class_id="0" tracking_level="0" version="0">
	<category>navigation</category>
	<type>Localization</type>
	<sender_name>position_estimate</sender_name>
	<id>POSITION_ESTIMATE0</id>
	<tag></tag>
	<time>1293855841.815181</time>
	<hdr_version>0</hdr_version>
</XmlSerDesHeader>
</boost_serialization>

<?xml version="1.0" encoding="UTF-8" standalone="yes" ?>
<!DOCTYPE boost_serialization>
<boost_serialization signature="serialization::archive" version="5">
<Localization class_id="0" tracking_level="0" version="0">
	<time>1293855850.494473</time>
	<coordinate></coordinate>
	<pos_x>16564.051</pos_x>
	<pos_y>-33475.105</pos_y>
	<pos_the>0.00040870431</pos_the>
	<pos_err_x>0.18618178</pos_err_x>
	<pos_err_y>0.17212869</pos_err_y>
	<pos_err_the>1.2774187</pos_err_the>
	<vel_x>0</vel_x>
	<vel_y>0</vel_y>
	<vel_the>0</vel_the>
</Localization>
</boost_serialization>
</pre>

<h3>主なパラメータの解説</h3>

<h4>pos_x, pos_y, pos_the</h4>
<ul>
<li>自己位置 x[m], y[m], the[rad]
</ul>

その他のパラメータに関しては，実装中及び将来の拡張性のために用意している．

\page outputdata 出力データ

<h1>環境認識コンポーネント出力データ</h1>

<h2>概要</h2>
GridMap.h の構造体 GridMap をXMLにシリアライズして出力する．

<h2>出力データの例</h2>

<pre>
<?xml version="1.0" encoding="UTF-8" standalone="yes" ?>
<!DOCTYPE boost_serialization>
<boost_serialization signature="serialization::archive" version="5">
<XmlSerDesHeader class_id="0" tracking_level="0" version="0">
	<category>navigation</category>
	<type>GridMap</type>
	<sender_name>env_recognition</sender_name>
	<id>ENV_RECOGNITION0</id>
	<tag></tag>
	<time>1293872175.785228</time>
	<hdr_version>0</hdr_version>
</XmlSerDesHeader>
</boost_serialization>

<?xml version="1.0" encoding="UTF-8" standalone="yes" ?>
<!DOCTYPE boost_serialization>
<boost_serialization signature="serialization::archive" version="5">
<GridMap class_id="0" tracking_level="0" version="0">
	<time>1293872177.566797</time>
	<id></id>
	<coordinate>self</coordinate>
	<area_x0>-1</area_x0>
	<area_y0>-2</area_y0>
	<area_x1>3</area_x1>
	<area_y1>2</area_y1>
	<size_x>0.050000001</size_x>
	<size_y>0.050000001</size_y>
	<range_min>0</range_min>
	<range_max>1</range_max>
	<range_step>0.1</range_step>
	<map class_id="1" tracking_level="0" version="0">
		<range_min>0</range_min>
		<range_max>1</range_max>
		<n_values_>10</n_values_>
		<n_elems>6400</n_elems>
		<n_bits>4</n_bits>
		<packed_bytes class_id="2" tracking_level="0" version="0">
			<n_bytes>3200</n_bytes>
			<byte_stream>
REREREREREQUFBQUFBREFBQURBQUFEQUFBREFBERQRERQUFBEUFBQUQRERERERERERERERER

　　　　　　　　　　　　　　＜途中省略＞

ERERERFEREREREQUERERERERERERERERERERERQRERERERERERERERERERERQUREREREERER
ERERERERERERERERERE=
			</byte_stream>
		</packed_bytes>
	</map>
</GridMap>
</boost_serialization>

</pre>

<h2>主なパラメータの解説</h2>

<h4>area_x0, area_y0, area_x1, area_y1</h4>
<ul>
<li>存在確率を表すグリッドマップの領域 x0, y0, x1, y1 [m]
</ul>

<h4>size_x, size_y</h4>
<ul>
<li>存在確率を表すグリッドマップのグリッドのサイズ x, y [m] 
</ul>

<h4>range_min, range_max, range_step</h4>
<ul>
<li>存在確率を表すグリッドマップの最大値，最小値，ステップ 
<li>グリッドマップのデータを圧縮するため．
<li>0.0,1.0,0.1の場合は11段階のため，4ビットのデータとして扱われる．
</ul>

<h4>byte_stream</h4>
<ul>
<li>存在確率を表すグリッドマップ
<li>グリッドマップのデータを上記のように圧縮した後，Base64を用いてテキストに変換した値
</ul>

その他のパラメータに関しては，実装中及び将来の拡張性のために用意している．

 */

#include <rtm/Manager.h>
#include <iostream>
#include <string>
#include "EnvRecognition.h"


void MyModuleInit(RTC::Manager* manager)
{
  EnvRecognitionInit(manager);
  RTC::RtcBase* comp;

  // Create a component
  comp = manager->createComponent("EnvRecognition");


  // Example
  // The following procedure is examples how handle RT-Components.
  // These should not be in this function.

  // Get the component's object reference
//  RTC::RTObject_var rtobj;
//  rtobj = RTC::RTObject::_narrow(manager->getPOA()->servant_to_reference(comp));

  // Get the port list of the component
//  PortList* portlist;
//  portlist = rtobj->get_ports();

  // getting port profiles
//  std::cout << "Number of Ports: ";
//  std::cout << portlist->length() << std::endl << std::endl;
//  for (CORBA::ULong i(0), n(portlist->length()); i < n; ++i)
//  {
//    Port_ptr port;
//    port = (*portlist)[i];
//    std::cout << "Port" << i << " (name): ";
//    std::cout << port->get_port_profile()->name << std::endl;
//
//    RTC::PortInterfaceProfileList iflist;
//    iflist = port->get_port_profile()->interfaces;
//    std::cout << "---interfaces---" << std::endl;
//    for (CORBA::ULong i(0), n(iflist.length()); i < n; ++i)
//    {
//      std::cout << "I/F name: ";
//      std::cout << iflist[i].instance_name << std::endl;
//      std::cout << "I/F type: ";
//      std::cout << iflist[i].type_name << std::endl;
//      const char* pol;
//      pol = iflist[i].polarity == 0 ? "PROVIDED" : "REQUIRED";
//      std::cout << "Polarity: " << pol << std::endl;
//    }
//    std::cout << "---properties---" << std::endl;
//    NVUtil::dump(port->get_port_profile()->properties);
//    std::cout << "----------------" << std::endl << std::endl;
//  }

  return;
}

int main (int argc, char** argv)
{
  RTC::Manager* manager;
  manager = RTC::Manager::init(argc, argv);

  // Initialize manager
  manager->init(argc, argv);

  // Set module initialization proceduer
  // This procedure will be invoked in activateManager() function.
  manager->setModuleInitProc(MyModuleInit);

  // Activate manager and register to naming service
  manager->activateManager();

  // run the manager in blocking mode
  // runManager(false) is the default.
  manager->runManager();

  // If you want to run the manager in non-blocking mode, do like this
  // manager->runManager(true);

  return 0;
}

