#include <iostream>
#include <string>
#include <sstream>

#include <sys/time.h>

#include "EnvRecognitionConfig.h"

// XML パーサ用
#include <xercesc/sax2/SAX2XMLReader.hpp>
#include <xercesc/sax2/XMLReaderFactory.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include "XmlHandler.h"

using namespace std;

EnvRecognitionConfig::
EnvRecognitionConfig():
	category(""),
	name    (""),
	id      ("")
{
	input_number = 0;
	gui = false;
}

EnvRecognitionConfig::
~EnvRecognitionConfig()
{
}


// TODO: 送信側 (コンフィグ保存) の実装は必要？
int EnvRecognitionConfig::
SaveToFile(string filename)
{
	return 0;
}

int EnvRecognitionConfig::
LoadFromFile(string filename)
{

	vector<string> str;
	
	InitXmlParser();

	if (!XmlParser(filename, "rtc_config", "name", &str)){
		cerr << "Cannot find [name] in configuration file" << endl;
		return -1;
	}
	name = str[0];

	if (!XmlParser(filename, "rtc_config", "category", &str)){
		cerr << "Cannot find [category] in configuration file" << endl;
		return -1;
	}
	category = str[0];

	if (!XmlParser(filename, "id", "", &str)){
		cerr << "Cannot find <id> in configuration file" << endl;
		return -1;
	}
	id = str[0];

	if (!XmlParser(filename, "receive", "id", &str)){
		cerr << "Cannot find <receive> in configuration file" << endl;
		return -1;
	}
	receive = str;

	if (!XmlParser(filename, "input_number", "", &str)){
		cerr << "Cannot find <input_number> in configuration file" << endl;
		return -1;
	}
	sscanf(str[0].c_str(), "%d", &input_number);

	if (!XmlParser(filename, "timeout", "", &str)){
		cerr << "Cannot find <timeout> in configuration file" << endl;
		return -1;
	}
	sscanf(str[0].c_str(), "%lf", &timeout);

	if (!XmlParser(filename, "work_area", "", &str)){
		cerr << "Cannot find <work_area> in configuration file" << endl;
		return -1;
	}
	sscanf(str[0].c_str(), "%f,%f,%f,%f", &work_map.area_x0, &work_map.area_y0, &work_map.area_x1, &work_map.area_y1);

	if (!XmlParser(filename, "work_grid_size", "", &str)){
		cerr << "Cannot find <work_grid_size> in configuration file" << endl;
		return -1;
	}
	sscanf(str[0].c_str(), "%f,%f", &work_map.size_x, &work_map.size_y);

	if (!XmlParser(filename, "area", "", &str)){
		cerr << "Cannot find <area> in configuration file" << endl;
		return -1;
	}
	sscanf(str[0].c_str(), "%f,%f,%f,%f", &grid_map.area_x0, &grid_map.area_y0, &grid_map.area_x1, &grid_map.area_y1);

	if (!XmlParser(filename, "grid_size", "", &str)){
		cerr << "Cannot find <grid_size> in configuration file" << endl;
		return -1;
	}
	sscanf(str[0].c_str(), "%f,%f", &grid_map.size_x, &grid_map.size_y);

	if (!XmlParser(filename, "range", "", &str)){
		cerr << "Cannot find <range> in configuration file" << endl;
		return -1;
	}
	sscanf(str[0].c_str(), "%f,%f,%f", &grid_map.range_min, &grid_map.range_max, &grid_map.range_step);
	sscanf(str[0].c_str(), "%f,%f,%f", &work_map.range_min, &work_map.range_max, &work_map.range_step);

	if (!XmlParser(filename, "memory_period", "", &str)){
		cerr << "Cannot find <memory_period> in configuration file" << endl;
		return -1;
	}
	sscanf(str[0].c_str(), "%f", &memory_period);

	if (!XmlParser(filename, "memory_decrease", "", &str)){
		cerr << "Cannot find <memory_decrease> in configuration file" << endl;
		return -1;
	}
	sscanf(str[0].c_str(), "%f", &memory_decrease);

	if (!XmlParser(filename, "gui", "", &str)){
		cerr << "Cannot find <gui> in configuration file" << endl;
		return -1;
	}
	if (str[0] == "ON"){
		gui = true;
	} else{
		gui = false;
	}

	TerminateXmlParser();

	return 0;
}

std::ostream&
operator<<(std::ostream& os, const EnvRecognitionConfig& urg)
{
	int i;

	os << "[EnvRecgnitionConfig]" << fixed << endl;
	os << "category:       " << urg.category << endl;
	os << "name:           " << urg.name << endl;
	os << "id:             " << urg.id << endl;
	os << "input_number:   " << urg.input_number << endl;
	os << "timeout:        " << urg.timeout << endl;
	os << "memory_period:  " << urg.memory_period << endl;
	os << "memory_decrease " << urg.memory_decrease << endl;
	os << "gui:            " << urg.gui << endl;

	os << "receive: ";
	for(i=0; i<(int)urg.receive.size(); i++){
		if(i != 0){
			os << ", ";
		}
		os << urg.receive[i];
	}
	os << endl;

	return os;
}

