#include "EnvRecognitionDialog.h"
#include "EnvRecognitionDialog_int.h"

#include <QMainWindow>

#include "GridMap.h"

pthread_mutex_t dialog_mutex;
QApplication *app = NULL;
int data_x = 0, data_y = 0;
float *draw_data = NULL;

EnvRecognitionWidget::
EnvRecognitionWidget(QWidget *parent): QWidget(parent)
{
	setWindowTitle(tr("EnvRecognition Dialog"));
	setPalette(QPalette(QColor(255, 255, 255)));
	setAutoFillBackground(true);
	setToolTip(tr("EnvRecognition\n[in]GridMap Sensor\n[in]Localization Position\n[out]GridMap Merged Data"));
}

void EnvRecognitionWidget::
set_value(GridMap *grid_map)
{
	pthread_mutex_lock(&dialog_mutex);

	int x, y;
	x = (int)((grid_map->area_x1 - grid_map->area_x0 + grid_map->size_x / 2.0f) / grid_map->size_x);
	y = (int)((grid_map->area_y1 - grid_map->area_y0 + grid_map->size_y / 2.0f) / grid_map->size_y);

	if ((data_x != x)||(data_y != y)){
		if (draw_data != NULL) free(draw_data);
		data_x = x;
		data_y = y;
		draw_data = (float *)malloc(x * y * sizeof(float));
	}
	for(int i = 0; i < (x * y); i ++){
		draw_data[i] = grid_map->map[i];
	}
	
	pthread_mutex_unlock(&dialog_mutex);
	update();
}

void EnvRecognitionWidget::
paintEvent(QPaintEvent *event)
{
	QPainter painter(this);
	painter.setPen(Qt::NoPen);

	int x0, y0, x1 = 0, y1 = 0, red, green, blue;
	float rx, ry;
	int no = 0;
	pthread_mutex_lock(&dialog_mutex);
	rx = (float)rect().width()  / data_x;
	ry = (float)rect().height() / data_y;
	for(int y = 0; y < data_y; y ++){
		for(int x = 0; x < data_x; x ++){
			x0 = (int)((x  ) * rx), y0 = (int)((y  ) * ry);
			x1 = (int)((x+1) * rx), y1 = (int)((y+1) * ry);
			red   = 0xff * draw_data[no];
			green = 0xff * draw_data[no];
			blue  = 0xff * draw_data[no];
			no ++;
			painter.setBrush(QColor(red, green, blue, 255));
			painter.drawRect(x0, y0, (int)rx+1, (int)ry+1);
		}
	}
	pthread_mutex_unlock(&dialog_mutex);
}

EnvRecognitionWidget *envRecognitionWidget = NULL;

void*
windowThread(void* param)
{
	int argc = 1;
	char *argv[] = {};

	pthread_mutex_init( &dialog_mutex, NULL );
	app = new QApplication(argc, argv);
	envRecognitionWidget = new EnvRecognitionWidget();
	envRecognitionWidget->show();
	app->exec();

	delete envRecognitionWidget;
	delete app;

	return 0;
}

int
initialize_window(void)
{
	return 0;
}

void
close_window(void)
{
	app->quit();

	return;
}

int
set_value(GridMap *grid_map)
{
	if (envRecognitionWidget != NULL){
		envRecognitionWidget->set_value(grid_map);
	}

	return 0;
}

