#ifndef POTENTIAL_H
#define POTENTIAL_H

#include <string>
#include <vector>
#include <iostream>

#include "GridMap.h"
#include "Path.h"
#include "Localization.h"
#include "LocalPathPlanConfig.h"

using namespace std;

// CPotential
/** 
 */
class CPotential
{
public:
	CPotential();
	virtual ~CPotential();

	// ■ 初期化 ■■■■■
	int init();

	// ■ 設定 ■■■■■
	int setConfiguration(LocalPathPlanConfig *conf);

	// ■ 入力 ■■■■■
	// 衝突リスクを加味した障害物グリッドマップ
	int setGridMap(GridMap *gmap);

	// 大経路入力
	int setGlobalPath(Path *gpath);

	// 現在の位置の入力
	int setLocalization(Localization *localization);

	// ポテンシャルの傾き
	int setSlope(float sl);
	
	// ■ 出力 ■■■■■
	// 経路データ
	int getLocalPath(Path *lpath);

	// ポテンシャルグリッドマップ
	int getPotentialMap(GridMap *pmap);

	// ■ 終了処理 ■■■■
	int close();

private:
	// ポテンシャルマップの計算
	int calculatePotentialMap();

	// ポテンシャルに基づくパスの計算
	int calculatePath();

	GridMap grid_map; ///< リスクマップ（存在確率マップ）
	GridMap potential_map; ///< ポテンシャルマップ

	Path global_path; ///< 第９系の座標系の経路（大経路モジュールから受信するデータ）
	Path dist_path;   ///< global_pathから経由したパスを除いた経路
	Path local_path;  ///< ロボット座標系で移動する経路及び目標速度ベクトル

	Localization loc;

	float slope; ///< ポテンシャルの傾き (/m)
	float goal_radius; ///< ゴールの半径
	bool update; ///< 出力時に必要に応じてupdateするため
};

#endif // POTENTIAL_H
